﻿using UnityEngine;
using System.Collections;

public class Personagem : MonoBehaviour
{
	public static Personagem Instance
	{
		get
		{
			if (m_instance == null)
			{
				m_instance = GameObject.FindObjectOfType<Personagem>();
				DontDestroyOnLoad(m_instance.gameObject);
			}

			return m_instance;
		}
	}

	public Vector2[] tipoTrajetoria;
	public GameObject telaInicio;
	public GameObject telaGameOver;

	int tipoAtual = 0;
	static Personagem m_instance;
	Vector3 vetorTopo;
	Vector3 vetorFundo;
	Vector3 posicaoOriginal;
	bool estaMovendoPraCima = false;
	bool parado = true;
	bool atualizarSenoideTopo = false;
	bool atualizarSenoideFundo = false;
	new Rigidbody2D rigidbody2D;
	bool morreu = false;

	// Use this for initialization
	void Start ()
	{
		rigidbody2D = GetComponent<Rigidbody2D>();
		posicaoOriginal = transform.position;

		Reiniciar();
	}

	public void Reiniciar()
	{
		morreu = false;
		transform.position = posicaoOriginal;
		vetorTopo = new Vector3( posicaoOriginal.x,  posicaoOriginal.y + tipoTrajetoria[tipoAtual].x, posicaoOriginal.z );
		vetorFundo = new Vector3( posicaoOriginal.x,  posicaoOriginal.y - tipoTrajetoria[tipoAtual].x, posicaoOriginal.z );
	}

	public void ReiniciarJogo()
	{
		telaGameOver.SetActive( false );
		telaInicio.SetActive( true );

		InimigoController.Instance.LimparInimigos();
		CenarioController.Instance.ReiniciarCenarios();
		RitmoControle.Instance.Reiniciar();

		Reiniciar();
	}
	
	// Update is called once per frame
	void Update ()
	{
		if ( !morreu )
		{
			if ( estaMovendoPraCima )
			{
				if ( transform.position.y >= vetorTopo.y - 0.05f )
				{
					// Move pra baixo
					iTween.MoveTo( gameObject, iTween.Hash( "position", new Vector3( vetorFundo.x, vetorFundo.y, vetorFundo.z ), "time", tipoTrajetoria[tipoAtual].y, "easetype", iTween.EaseType.easeInOutSine ) );
					estaMovendoPraCima = false;

					if ( atualizarSenoideTopo )
					{
						vetorTopo = new Vector3( posicaoOriginal.x, posicaoOriginal.y + tipoTrajetoria[tipoAtual].x, posicaoOriginal.z );
						atualizarSenoideTopo = false;
					}
				}
			}
			else
			{
				if ( transform.position.y <= vetorFundo.y + 0.05f )
				{
					// Move pra cima
					iTween.MoveTo( gameObject, iTween.Hash( "position", new Vector3( vetorTopo.x, vetorTopo.y, vetorTopo.z ), "time", tipoTrajetoria[tipoAtual].y, "easetype", iTween.EaseType.easeInOutSine ) );
					estaMovendoPraCima = true;

					if ( atualizarSenoideFundo )
					{
						vetorFundo = new Vector3( posicaoOriginal.x, posicaoOriginal.y - tipoTrajetoria[tipoAtual].x, posicaoOriginal.z );
						atualizarSenoideFundo = false;
					}

				}
			}
		}
		else
		{
			if ( transform.position.y < -10 )
			{
				transform.position = new Vector3( transform.position.x, -10, transform.position.z );
				rigidbody2D.isKinematic = true;
			}
		}
	}

	public void mudarTrajetoria( int tipo )
	{		
		tipoAtual = tipo;
		atualizarSenoideTopo = true;
		atualizarSenoideFundo = true;

		if ( estaMovendoPraCima  )
		{
			vetorFundo = new Vector3( posicaoOriginal.x, posicaoOriginal.y - tipoTrajetoria[tipo].x, posicaoOriginal.z );
			atualizarSenoideFundo = false;
		}
		else
		{
			vetorTopo = new Vector3( posicaoOriginal.x, posicaoOriginal.y + tipoTrajetoria[tipo].x, posicaoOriginal.z );
			atualizarSenoideTopo = false;
		}

		if ( iTween.Count( gameObject ) < 1 )
		{
			vetorFundo = new Vector3( posicaoOriginal.x, posicaoOriginal.y - tipoTrajetoria[tipo].x, posicaoOriginal.z );
			vetorTopo = new Vector3( posicaoOriginal.x, posicaoOriginal.y + tipoTrajetoria[tipo].x, posicaoOriginal.z );
			iTween.MoveTo( gameObject, iTween.Hash( "position", new Vector3( vetorTopo.x, vetorTopo.y, vetorTopo.z ), "time", tipoTrajetoria[tipoAtual].y, "easetype", iTween.EaseType.easeInOutSine ) );
			estaMovendoPraCima = true;
		}
	}

	public void DestroyTudo( GameObject o )
	{
		Destroy( o, 0.1f );
	}

	void OnTriggerEnter2D( Collider2D other ) 
	{
		if ( other.CompareTag( "Inimigo" ) )
		{
			morreu = true;
			iTween.Stop( gameObject );
			rigidbody2D.isKinematic = false;
			telaGameOver.SetActive( true );

			SpawnerController.Instance.GameOver();
			RitmoControle.Instance.GameOver();
			InimigoController.Instance.PararInimigos();
			CenarioController.Instance.PararCenario();
		}
	}
}
