﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class SpawnerController : MonoBehaviour
{
	public static SpawnerController Instance
	{
		get
		{
			if (m_instance == null)
			{
				m_instance = GameObject.FindObjectOfType<SpawnerController>();
				DontDestroyOnLoad(m_instance.gameObject);
			}

			return m_instance;
		}
	}

	public Vector3 posicaoCima;
	public Vector3 posicaoMeio;
	public Vector3 posicaoBaixo;
	public GameObject enemyPrefab;
	public GameObject enemyHierarquia;
	public bool podeSpawnar;

	static SpawnerController m_instance;
	Sequencia primeiraOnda;
	Sequencia segundaOnda;
	Sequencia ondaAtual;
	bool temDeSpawnar;
	float temporizador;
	float temporizadorSequencia;
	float tempoUltimoSpawn;
	int itemAtual;

	// Use this for initialization
	void Start () {
		primeiraOnda = new Sequencia();

		primeiraOnda.Registrar( Sequencia.POSICAO_BAIXO, 0f );
		primeiraOnda.Registrar( Sequencia.POSICAO_CIMA, 1f );
		primeiraOnda.Registrar( Sequencia.POSICAO_BAIXO, 1f );
		primeiraOnda.Registrar( Sequencia.POSICAO_CIMA, 1f );

		segundaOnda = new Sequencia();

		segundaOnda.Registrar( Sequencia.POSICAO_MEIO, 0f );
		segundaOnda.Registrar( Sequencia.POSICAO_MEIO, 1.8f );
		segundaOnda.Registrar( Sequencia.POSICAO_MEIO, 1.8f );

		podeSpawnar = false;
	}

	public void Reiniciar()
	{
		podeSpawnar = true;
		temDeSpawnar = false;
		tempoUltimoSpawn = 0f;
		itemAtual = 0;
		temporizador = 0f;
		ondaAtual = primeiraOnda;
	}

	public void GameOver()
	{
		podeSpawnar = false;
	}
	
	// Update is called once per frame
	void Update()
	{
		if ( !podeSpawnar )
			return; 
		
		temporizador += Time.deltaTime;

		if ( temporizador > RitmoControle.Instance.tempoCompasso )
		{
			temDeSpawnar = true;
			tempoUltimoSpawn = 0f;
			itemAtual = 0;
			temporizador = 0f;

			// TODO: Escolher onda aleatoriamente
			ondaAtual = Random.Range( 0, 2 ) == 0 ? primeiraOnda : segundaOnda;
		}

		if ( temDeSpawnar )
		{
			tempoUltimoSpawn += Time.deltaTime;

			if ( tempoUltimoSpawn > ondaAtual.PegarTempo( itemAtual ) )
			{
				GameObject obj = Instantiate( enemyPrefab );
				obj.transform.parent = enemyHierarquia.transform;

				if ( ondaAtual.PegarPosicao( itemAtual ) == Sequencia.POSICAO_CIMA )
				{
					obj.transform.position = posicaoCima;
				}

				if ( ondaAtual.PegarPosicao( itemAtual ) == Sequencia.POSICAO_MEIO )
				{
					obj.transform.position = posicaoMeio;
				}

				if ( ondaAtual.PegarPosicao( itemAtual ) == Sequencia.POSICAO_BAIXO )
				{
					obj.transform.position = posicaoBaixo;
				}

				itemAtual++;
				tempoUltimoSpawn = 0;

				if ( itemAtual >= ondaAtual.Count )
					temDeSpawnar = false;
			}
		}
	}

	public void DestroyTudo( GameObject o )
	{
		Destroy( o, 0.1f );
	}

	private class Sequencia
	{
		public const int POSICAO_CIMA = 0;
		public const int POSICAO_MEIO = 1;
		public const int POSICAO_BAIXO = 2;

		List<int> posicoes;
		List<float> tempos;

		public Sequencia()
		{
			posicoes = new List<int>();
			tempos = new List<float>();
		}

		public void Registrar( int pos, float tempoDelay )
		{
			posicoes.Add( pos );
			tempos.Add( tempoDelay );
		}

		public float PegarPosicao( int i )
		{
			return posicoes[i];
		}

		public float PegarTempo( int i )
		{
			return tempos[i];
		}

		public int Count
		{
			get { return posicoes.Count; }
		}
	}
}
